"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = exports.getPropertyName = exports.getStylesForProperty = exports.transformRawValue = void 0;

var _postcssValueParser = _interopRequireDefault(require("postcss-value-parser"));

var _camelize = _interopRequireDefault(require("camelize"));

var _index = _interopRequireDefault(require("./transforms/index"));

var _devPropertiesWithoutUnitsRegExp = _interopRequireDefault(require("./devPropertiesWithoutUnitsRegExp"));

var _TokenStream = _interopRequireDefault(require("./TokenStream"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

// Note if this is wrong, you'll need to change tokenTypes.js too
var numberOrLengthRe = /^([+-]?(?:\d*\.)?\d+(?:e[+-]?\d+)?)(?:px)?$/i;
var numberOnlyRe = /^[+-]?(?:\d*\.\d*|[1-9]\d*)(?:e[+-]?\d+)?$/i;
var boolRe = /^true|false$/i;
var nullRe = /^null$/i;
var undefinedRe = /^undefined$/i; // Undocumented export

var transformRawValue = function transformRawValue(propName, value) {
  if (process.env.NODE_ENV !== 'production') {
    var needsUnit = !_devPropertiesWithoutUnitsRegExp["default"].test(propName);
    var isNumberWithoutUnit = numberOnlyRe.test(value);

    if (needsUnit && isNumberWithoutUnit) {
      // eslint-disable-next-line no-console
      console.warn("Expected style \"".concat(propName, ": ").concat(value, "\" to contain units"));
    }

    if (!needsUnit && value !== '0' && !isNumberWithoutUnit) {
      // eslint-disable-next-line no-console
      console.warn("Expected style \"".concat(propName, ": ").concat(value, "\" to be unitless"));
    }
  }

  var numberMatch = value.match(numberOrLengthRe);

  if (numberMatch !== null) {
    var num = Number(numberMatch[1]);

    if (/px/.test(value)) {
      return "scalePx2dp(".concat(num, ")");
    } else {
      return num;
    }
  }

  var boolMatch = value.match(boolRe);
  if (boolMatch !== null) return boolMatch[0].toLowerCase() === 'true';
  var nullMatch = value.match(nullRe);
  if (nullMatch !== null) return null;
  var undefinedMatch = value.match(undefinedRe);
  if (undefinedMatch !== null) return undefined;
  return value;
};

exports.transformRawValue = transformRawValue;

var baseTransformShorthandValue = function baseTransformShorthandValue(propName, value) {
  var ast = (0, _postcssValueParser["default"])(value);
  var tokenStream = new _TokenStream["default"](ast.nodes);
  return _index["default"][propName](tokenStream);
};

var transformShorthandValue = process.env.NODE_ENV === 'production' ? baseTransformShorthandValue : function (propName, value) {
  try {
    return baseTransformShorthandValue(propName, value);
  } catch (e) {
    throw new Error("Failed to parse declaration \"".concat(propName, ": ").concat(value, "\""));
  }
};

var getStylesForProperty = function getStylesForProperty(propName, inputValue, allowShorthand) {
  var isRawValue = allowShorthand === false || !(propName in _index["default"]);
  var value = inputValue.trim();
  var propValues = isRawValue ? _defineProperty({}, propName, transformRawValue(propName, value)) : transformShorthandValue(propName, value);
  return propValues;
};

exports.getStylesForProperty = getStylesForProperty;

var getPropertyName = function getPropertyName(propName) {
  var isCustomProp = /^--\w+/.test(propName);

  if (isCustomProp) {
    return propName;
  }

  return (0, _camelize["default"])(propName);
};

exports.getPropertyName = getPropertyName;

var _default = function _default(rules) {
  var shorthandBlacklist = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
  return rules.reduce(function (accum, rule) {
    var propertyName = getPropertyName(rule[0]);
    var value = rule[1];
    var allowShorthand = shorthandBlacklist.indexOf(propertyName) === -1;
    return Object.assign(accum, getStylesForProperty(propertyName, value, allowShorthand));
  }, {});
};

exports["default"] = _default;